/* aeconfig.c -- Aexpl config file handling
 *
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 *
 * This file is part of the aexpl-ng distribution.
 *
 * The aexpl-ng distribution is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "aeconfig.h"

#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <string.h>

static xmlXPathObjectPtr _get_xobject (const xmlChar * path);
static inline void _cfg_missing (const char *path, char *buf, size_t size);
static int _cfg_read_custom_str (const char *path, const char *element,
				 char *dest);
static struct _cfg_custom_block_t *_cfg_read_custom_block (const char *path,
							   struct
							   _cfg_custom_block_t
							   *dest);

static xmlDocPtr cfg;

/* {{{ _get_xobject */
static xmlXPathObjectPtr
_get_xobject (const xmlChar * path)
{
  xmlXPathContextPtr context = xmlXPathNewContext (cfg);
  xmlXPathObjectPtr object = xmlXPathEvalExpression (path, context);

  xmlXPathFreeContext (context);

  return (xmlXPathNodeSetIsEmpty (object->nodesetval) ? NULL : object);
}

/* }}} */

/* {{{ _cfg_missing */
static inline void
_cfg_missing (const char *path, char *buf, size_t size)
{
  snprintf (buf, size, "Missing element: '%s'.", path);
}

/* }}} */

/* {{{ cfg_setup */
int
cfg_setup (const char *file)
{
  return ((cfg = xmlParseFile (file)) ? 1 : 0);
}

/* }}} */

/* {{{ cfg_teardown */
void
cfg_teardown (void)
{
  xmlFreeDoc (cfg);
  xmlCleanupParser ();
}

/* }}} */

/* {{{ cfg_read_str */
char *
cfg_read_str (const char *path)
{
  xmlXPathObjectPtr object = _get_xobject (path);
  xmlNodeSetPtr nodeset = object ? object->nodesetval : NULL;
  xmlChar *value;
  int value_size;
  char *cfg_str = NULL;

  if (nodeset && nodeset->nodeNr)
    {
      value =
	xmlNodeListGetString (cfg, nodeset->nodeTab[0]->xmlChildrenNode, 1);
      if (!value)
	return (NULL);

      value_size = strlen (value) + 1;
      cfg_str = (char *) calloc (value_size, sizeof (char));
      strncpy (cfg_str, value, value_size);

      xmlFree (value);
      xmlXPathFreeObject (object);
    }

  return (cfg_str);
}

/* }}} */

/* {{{ cfg_read_int */
int
cfg_read_int (const char *path)
{
  char *value = cfg_read_str (path);
  int cfg_int = !value
    || (sscanf (value, "%i", &cfg_int) != 1) ? -1 : cfg_int;

  free (value);
  return (cfg_int);
}

/* }}} */

/* {{{ cfg_read_bool */
int
cfg_read_bool (const char *path)
{
  char *value = cfg_read_str (path);
  short cfg_bool = !value ? -1 : (!strcmp (value, "true") ? 1 : 0);

  free (value);
  return (cfg_bool);
}

/* }}} */

/* {{{ _cfg_read_custom_str */
static int
_cfg_read_custom_str (const char *path, const char *element, char *dest)
{
  char path_buf[PATH_MAX];
  char *str_buf;

  snprintf (path_buf, PATH_MAX, "%s/%s", path, element);
  if ((str_buf = cfg_read_str (path_buf)))
    {
      strcpy (dest, str_buf);
      free (str_buf);

      return (1);
    }
  else
    return (0);
}

/* }}} */

/* {{{ _cfg_read_custom_block */
static struct _cfg_custom_block_t *
_cfg_read_custom_block (const char *path, struct _cfg_custom_block_t *dest)
{
  if (!_cfg_read_custom_str (path, "exploit_db", dest->exploit_db)
      || !_cfg_read_custom_str (path, "update_url", dest->update_url)
      || !_cfg_read_custom_str (path, "ca_cert", dest->ca_cert))
    return (NULL);

  dest->available = 1;
  return (dest);
}

/* }}} */

/* {{{ cfg_read_custom_block */
struct _cfg_custom_block_t *
cfg_read_custom_block (const char *path)
{
  struct _cfg_custom_block_t *value = (struct _cfg_custom_block_t *)
    malloc (sizeof (struct _cfg_custom_block_t));

  if (!_cfg_read_custom_block (path, value))
    {
      free (value);
      return (NULL);
    }

  return (value);
}

/* }}} */

/* {{{ cfg_read_elements */
int
cfg_read_elements (const cfg_element_t * list, char *err_buf, int buf_size)
{
  unsigned index = 0;
  cfg_element_t element = *list;
  long value;

  for (; (list + index)->path; ++index, element = *(list + index))
    {
      switch (element.type)
	{
	case etype_none:
	  return (0);

	case etype_str:
	  value = (long) cfg_read_str (element.path);

	  if (!value && !element.optional)
	    {
	      _cfg_missing (element.path, err_buf, buf_size);

	      return (0);
	    }

	  if (value)
	    {
	      strncpy ((char *) element.buf, (char *) value, element.size);
	      free ((char *) value);
	    }
	  break;

	case etype_int:
	  value = (long) cfg_read_int (element.path);

	  if ((value == -1) && !element.optional)
	    {
	      _cfg_missing (element.path, err_buf, buf_size);

	      return (0);
	    }

	  if (value)
	    *(int *) element.buf = (int) value;

	  break;

	case etype_bool:
	  value = (long) cfg_read_bool (element.path);

	  if ((value == -1) && !element.optional)
	    {
	      _cfg_missing (element.path, err_buf, buf_size);

	      return (0);
	    }

	  if (value)
	    *(short *) element.buf = (short) value;

	  break;

	case etype_custom_block:
	  value = (long) _cfg_read_custom_block (element.path, element.buf);

	  if (!value && !element.optional)
	    {
	      _cfg_missing (element.path, err_buf, buf_size);

	      return (0);
	    }

	  break;
	}
    }

  return (1);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
